# Feature Selection for Network Features
#
# Remove the features with low variance and select the most important ones based on Genetic Algorithms
# NOTE: this version contraints the search to a small space using a desirability function.

#
# Input: X, a MxN matrix with M observation/subjects and N network features
#        Y, a Mx1 matrix with the response variable/classes
# Output: a subset of the most important features
#
# Author: Nadia Moreira da Silva
#         Newcastle University
#         @email: nadiamoreirass@gmail.com
############################################################

rm(list = ls())

if(!require(caret)){install.packages("caret")}
if(!require(doParallel)){install.packages("doParallel")}
if(!require(dplyr)){install.packages("dplyr")}
if(!require(pROC)){install.packages("pROC")}
if(!require(randomForest)){install.packages("randomForest")}
if(!require(e1071)){install.packages("e1071")}
if(!require(desirability)){install.packages("desirability")}

############################################################
# Import Data
# Y, Classes
Y = c(rep("A",23), rep("B",28), rep("C",23))
# Acute patients classified as "A"
# Controls classified as "B"
# Chronic patients classified as "C"

# X, Network metrics
# X, a MxN matrix with M observation/subjects and N network features
# filename = file.choose(new = FALSE) to select the file manually
filename = "~/Data/network_metrics.csv"

X=read.csv(filename,header = 1)

# Normalization
data_sd = sapply(X, sd, na.rm = TRUE)
data_mean = sapply(X, mean, na.rm = TRUE)
X_norm = sweep(X,2,data_mean)
X_norm = sweep(X_norm,2,FUN = "/",data_sd)

# Remove network metrics with p-value > 0.05 as explained in the paper
filename = "~/Data/network_metrics_below_p0.05.csv"
features=read.csv(filename,header = 0,sep=",")
X_norm = X_norm[,colnames(X_norm) %in% features$V1]

# Select acute and chronic patients data
X_norm = X_norm[c(1:23,52:74),]
Y = Y[c(1:23,52:74)]
x.train = X_norm
y.train = Y
###################################################

registerDoParallel(20) # Registrer a parallel backend
getDoParWorkers() # check that there are 6 workers

# Desirability function: the optimum set of features is selected based on accuracy (maximize)
# and number of features (minimize)
rfGA2 <- rfGA
rfGA2$fitness_intern <- function (object, x, y, maximize, p) {
  Accuracy <- rfStats(object)[1]
  d_accu <- dMax(0, 99)
  d_Size <- dMin(1, p, 2)
  overall <- dOverall(d_accu, d_Size)
  D <- predict(overall, data.frame(Accuracy, ncol(x)))
  c(D = D, Accuracy = as.vector(Accuracy))
}
# GARF controls options
ga_ctrl <- gafsControl(functions = rfGA2,
                         method = "cv",
                         repeats = 5,
                         metric = c(internal = "D", external = "Accuracy"),
                         maximize = c(internal = TRUE, external = TRUE),
                         genParallel=TRUE, # Use parallel programming
                         allowParallel = TRUE)


set.seed(123)
lev <- c("A","C")
y.train = as.factor(y.train)

#train = x.train %>% mutate(Label = y.train)
system.time(rf_ga3 <- gafs(x = x.train, y = y.train,
                           iters = 100, # 100 generations of algorithm
                           popSize = 25, # population size for each generation
                           pcrossover = 0.8,
                           pmutation = 0.1,
                           selection = "gabin_tourSelection",
                           levels = lev,
                           gafsControl = ga_ctrl))


final <- rf_ga3$optVariables

# #
cat("Train set: Internal Accuracy: ", rf_ga3$internal$Accuracy[ga_results$optIter])
cat("Train set: External Accuracy: ", rf_ga3$external$Accuracy[ga_results$optIter])
cat("Train set: Averages Accuracy: ", rf_ga3$averages$Accuracy[ga_results$optIter])

# Save optimum set of features
final = as.data.frame(final)
write.csv(final, file = "~/Data/GARF_set.csv",col.names=NA,row.names=FALSE)
cat("Selected Features:", final) # print the optimum set of features

