function m = graph_theoretical_analysis(A)

%%%%%%% Graph Theoretical Analysis
% Aim: This function determines the following network measures for a weighted and undirected graph A:

%%% Basic - Degree, Av. Degree, Diameter, Eccentricity, Density;
%%% Segregation - Transitivity, Local Clustering Coeff, Av. Clust Coeff, Local Efficiency;
%%% Integration - Characteristic Path Length, Global Efficiency;
%%% Resilience - Assortativity Coeff, Av. Neighbor Degree; Local Assortativity Coeff
%%% Centrality - Closeness, Betweeness, Eigenvector, Page Ranks

% Input: A is a connectivity matrix
% Output: Struct with all the calculated metrics

% NOTE: To run this function you need Brain Connectivity toolbox. Add it to
% your main script as in the following example:
% addpath /Users/nadia/Softwares/MATLAB/Toolboxes/BCT/2017_01_15_BCT

% Ndia Moreira da Silva, Newcastle University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Graph Representation
A_graph = graph(A);
D_graph = distance_wei(A);

%% Basic
% Degree
m.deg = degrees_und(A);

% Av. Degree
m.av_deg = mean(m.deg);

% Diameter % % Eccentricity
[m.L,m.E_global,m.ecc,m.radius, m.diameter] = charpath(D_graph);

% Density
[m.density] = density_und(A);

% Strength
[m.strength] = strengths_und(A);


%% Segregation
% Transitivity
m.T = transitivity_wu(A);

% Local Clust.Coeff
m.C = clustering_coef_wu(A);

% Av. Clust. Coeff
m.av_C = mean(m.C);

% Local Efficiency
m.Eloc = efficiency_wei(A,2);

%% Integration
% Characteristic Path Length
% m.L

% Global Efficiency
% m.E_global
m.E_global = efficiency_wei(A,0);


%% Resilience
% Assortativity Coeff
m.Assortativity = assortativity_wei(A,0);

% Av. Neighbor Degree
m.Knn = av_neighbor_degree(A);

% Local Assortativity Coeff
m.Assortativity_Local = local_assortativity_wu_sign(A);


%% Centrality
% Closeness
m.Closeness = centrality(A_graph,'closeness','Cost',A_graph.Edges.Weight);

% Betweeness
m.Betweenness = centrality(A_graph,'betweenness','Cost',A_graph.Edges.Weight);

% Eigenvector
m.Eigenvector = eigenvector_centrality_und(A);

% Page Ranks
m.PRank = pagerank_centrality(A, 0.85);

end
