# PCA analysis
#
# Apply PCA to cognitive scores to find the component(s) that best represent the variance in the clinical data
#
# Input: Y, a csv file with M subjects x N cognitive scores (headers included)
# Output: Principal component(s), plots
#
# Author: Nadia Moreira da Silva
#         Newcastle University
#         @email: nadiamoreirass@gmail.com
############################################################
rm(list = ls())
library(caret)
library(moments)
library(AID)
library("devtools")
install_github("kassambara/factoextra")
library("factoextra")
############################################################
# Functions
# skewness.significant is based on Pag 41 of the following book:
# https://books.google.co.uk/books?redir_esc=y&hl=pt-PT&id=LhEPlzgwRdQC&q=kurtosis#v=snippet&q=kurtosis&f=false
skewness.significant <- function (dataset){
  n = nrow(dataset)
  se.skewness = sqrt( (6*n*(n-1)) / ((n-2)*(n+1)*(n+3)) )
  skewnessPerColumn = skewness(dataset) / se.skewness 
  return(skewnessPerColumn)
}

############################################################
# Import Data
# Load Y
#filename = file.choose(new = FALSE) # select file manually
filename="~/Data/scores.csv"
Y=read.csv(filename,header = 1)

# PCA is sensible to skewed data.
# In the presence of skewed data you can 
# 1) transform the skewed features using BoxCox transformation 
# 2) use RobustPCA.
# In this study we transformed skewed features/scores with BoxCox.
# For more details on skewed data in PCA read the following paper:
# Hubert, M., Rousseeuw, P., & Verdonck, T. (2009). Robust PCA for skewed data and its outlier map. Computational Statistics & Data Analysis, 53(6), 2264-2274.

# Transform scores highly skewed 
skewnessValues = as.data.frame(skewness.significant(Y))
indSkewFeatures = which(abs(skewnessValues[,1]) >=1.96)

# BoxCox transformation
for (i in c(1:length(indSkewFeatures))){
  score = Y[,indSkewFeatures[i]]+1
  new_score=boxcoxnc(score, method = "sw", lambda = seq(-4,4,0.01), plot = TRUE)
  Y[,indSkewFeatures[i]]=new_score$tf.data
}

# run PCA 
scores.pca <- prcomp(Y,center = TRUE, scale = TRUE) 
print(scores.pca)
summary(scores.pca)
write.csv(score,'~Data/CFC_score.csv', row.names = FALSE)
# Idenitfy number of PC components to keep
# 
# Scree plot 
# use the plot to identify how many components you should keep: all components before the gap/elbow
fviz_eig(scores.pca,barfill = "#CCCCCC",barcolor = "#CCCCCC", linecolor = "black" )
score=scores.pca$x[,1] # only the first component was found before the gap

# to check the contribution of each score for the 1st component:
# scores.pca$rotation[,1]

# Extra Plots 
# Plot the variance explained of PC1 vs PC2 for each group 
groups_ind = c(rep("Acute",23),rep("Control",28), rep("Chronic",23))

library(devtools)
install_github("ggbiplot", "vqv")
library(ggbiplot)

g <- ggbiplot(scores.pca, obs.scale = 1, var.scale = 1, 
              groups = groups_ind, ellipse = TRUE, 
              circle = TRUE)
g <- g + scale_color_discrete(name = '')
g <- g + theme(legend.direction = 'horizontal', 
               legend.position = 'top')

g <- g +
  theme(
    panel.background = element_rect(fill = "#F5E3ED") # bg of the panel
    , plot.background = element_rect(fill = "transparent") # bg of the plot
    , panel.grid.major = element_line(colour = "grey",size=0.5)  # get rid of major grid
    , panel.grid.minor = element_blank() # get rid of minor grid
    , legend.background = element_rect(fill = "transparent") # get rid of legend bg
  )


print(g)

#  VIOLIN plot with both groups
X = data.frame(score = score, Group = as.factor(groups_ind))
library(ggplot2)

p<-ggplot(X, aes(x=Group, y=score, fill=Group)) +
  geom_violin(trim=TRUE) + geom_boxplot(width=0.1, fill="white") +
  labs(x="Group", y = "PCA score")
p + scale_fill_brewer(palette="Blues") + theme_classic()


