# Linear regression
#
# Fit a linear regresion model to explain the changes in the cognitive function over time with changes in structural connectivity
#
# Input: Y, csv file M subjects x 1 from the PCA analysis, CFC component
#        X, csv file M subjects x N network metrics from the graph theoretical analysis
#        Z, csv file with the list of network metrics selected by GARF
# Output: Performane of linear regression model (nRMSE, Adjusted-R2, Residual analysis)
#
# Author: Nadia Moreira da Silva
#         Newcastle University,
#         @email: nadiamoreirass@gmail.com
############################################################
############################################################
rm(list = ls())
library(caret)
library(foreign)
library(MASS)
source("~/Code/bootstrap.model.R")
############################################################
# Import Data

# Load Y, csv file with M subjects x 1 from the PCA analysis, CFC component
#filename = file.choose(new = FALSE) # select file manually
filename = "~/Data/CFC_score.csv"
Y=read.csv(filename,header = 1)

# Load X, csv file M subjects x N network metrics from the graph theoretical analysis
filename = "~/Data/network_metrics.csv"
X=read.csv(filename,header = 1)

# Compute change [ Chronic - Acute ]
longY = as.data.frame(Y[c(52:74),1]-Y[c(1:23),1])
longX = as.data.frame(X[c(52:74),]-X[c(1:23),])  

# Standardize data
data_sd = sapply(longX, sd, na.rm = TRUE)
data_mean = sapply(longX, mean, na.rm = TRUE)
X_norm = sweep(longX,2,data_mean)
X_norm = sweep(X_norm,2,FUN = "/",data_sd)

#Load Z, csv file with the list of network metrics selected by GARF
filename = "~/Data/GARF_set.csv"
features = read.csv(filename,header = 0,sep=",")

# Remove all network metrics not selected by GARF
XFiltered = X_norm[,colnames(X_norm) %in% features$V1]
XFiltered = XFiltered[ , ! apply( XFiltered , 2 , function(x) all(is.na(x)) ) ]

Y=as.data.frame(longY) # Transform longY to dataframe
label_p_value=TRUE
maxFeat=ncol(XFiltered)+1

set.seed(3)
while (label_p_value==TRUE){
  x=as.data.frame(cbind(XFiltered,Y))
  names(x)[ncol(x)]<-"Score" 
  # run linear regresion model
  j=lm(Score~.,x) 
  resultRegression = summary(j)
  print(summary(j))
  # check if residuals follow a normal distribution:
  print(shapiro.test(j$residuals))
  # Residuals analysis
  par(mfrow=c(2,3),bg="transparent",pch = 19,mai=c(0.6, 0.6, 0.6, 0.2))
  plot(as.matrix(Y),as.matrix(j$fitted.values),col ="#377eb8",xlab="True Score",cex.lab=0.7, ylab="Predicted Score",cex.axis=1)
  plot(as.matrix(Y),j$residuals,col ="#377eb8",xlab="True Score", ylab="Residuals",cex.lab=0.7,cex.axis=1)
  plot(as.matrix(j$fitted.values),j$residuals,col ="#377eb8",xlab="Predicted Score", ylab="Residuals",cex.lab=0.7,cex.axis=1)
  plot(x=c(tail(j$residuals,-1),j$residuals[1]),y=j$residuals,col ="#377eb8",xlab="X(k-1)", ylab="X(k)",cex.lab=0.7,cex.axis=1)
  hist(j$residuals,col ="#377eb8",border="white", xlab = "Residuals",cex.lab=0.7,cex.axis=1)
  qqnorm(j$residuals,col ="#377eb8",cex.lab=0.7,cex.main=0.1,cex.axis=1)
  qqline(j$residuals,col ="red")
  Obs=matrix(Y)
  Obs=Obs[[1]]
  RMSE_model = (RMSE(j$fitted.values,Obs))/(max(Y)-min(Y))
  
  # remove itertively the feature with higher p-value until all remaining features have p-value<0.05
  label_p_value = TRUE %in% (resultRegression$coefficients[-1,4]>=0.05)
  if (label_p_value==TRUE){
    ind_remove = which(resultRegression$coefficients[c(2:maxFeat),4]==max(resultRegression$coefficients[c(2:maxFeat),4]))
    maxFeat=maxFeat-1
    XFiltered = XFiltered[,-(ind_remove)]
    }
}


# Permutations

Ymat=matrix(Y)
Ymat=Ymat[[1]]
demo.boostrap <- bootstrap.model(Ymat, colnames(XFiltered), XFiltered, 1000)

mean.predicted = demo.boostrap$mean.predicted
sd.predicted = demo.boostrap$sd.predicted

# Plot Predicted change CFC vs True change in CFC
dataTest=data.frame("MeanPred" = mean.predicted[,1], "se" = sd.predicted[,1], "MeanObs" = as.numeric(row.names(mean.predicted)))
pd = position_dodge(.2) 
par(bg = 'transparent',mai=c(1.2,1,1,0.5))
g= ggplot(dataTest, aes(x=MeanObs, y=MeanPred)) +  xlab('True Change in CFC')+ ylab('Predicted Change CFC')+
  geom_point(color='#2980B9', size = 4)  +
  #geom_smooth(method=lm, color='#2C3E50') +
  geom_abline(intercept = 0.07962 , slope = 0.96299, colour='#2980B9',size=1) +
  geom_abline(intercept = 0, slope = 1, colour='red', linetype="dashed",size=1) +
  geom_errorbar(aes(ymin  = MeanPred - se,
                    ymax  = MeanPred + se),
                width = 0.1, 
                size  = 0.3, 
                position = pd,
                color = '#2980B9') +
  theme(
    axis.text=element_text(size=20,color="black"),
    axis.title=element_text(size=24),
    #panel.grid.major = element_blank(), 
    #panel.grid.minor = element_blank(),
    panel.background = element_rect(fill = "transparent",colour = NA),
    plot.background = element_rect(fill = "transparent",colour = NA),
    axis.line.x = element_line(color="black", size = 0.2),
    axis.line.y = element_line(color="black", size = 0.2)
  )
print(g)

#ggsave(g, filename = "~/Projects/Project@TBI/bootstrap_results.eps",  bg = "transparent")


